/*
 * Copyright (c) 2021 PIXILAB Technologies AB, Sweden (http://pixilab.se). All Rights Reserved.
 */

import * as feed from "../system_lib/Feed";
import {SimpleHTTP} from "../system/SimpleHTTP";
import {FeedDetailsSpec, FeedListSpec, ListData} from "../system_lib/Feed";
import {field} from "../system_lib/Metadata";


/**
 * A very simple feed, with a single collection instance. Returns al list of "visitors"
 * from a request to a Google spreadsheet. A single request provides all data. Hence, the
 * items returned from getList and getDetails are identical, and are cached from the
 * most recent getList call.
 */
export class VisitorsToday extends feed.Feed {
	constructor(env: feed.FeedEnv) {
		super(env);
		this.establishFeed(new Collection("Visitors"));
	}
}

const kSpreadsheetURL = "https://docs.google.com/spreadsheets/d/e/2PACX-1vQVE3v1sikxm1esuNdlW-erWrtA_GemjtKVJtmd0fpXQmux7AS8_7yFgewLKj99IY0NS5qz88NgJ8go/pub?gid=0&single=true&output=csv"

class Collection implements feed.StaticFeed<Item, Item> {
	readonly name: string;	// Our internal (short) name of this feed
	readonly collectionName: string;	// Official name of feed

	readonly listType = Item;
	readonly itemType = Item;

	private lastItems: Item[];	// Items fetched most recently through getList

	constructor(feedName: string) {
		this.name = feedName;
	}

	/**
	 * Get a list of items from the spreadsheet.
	 */
	getList(spec:feed.FeedListSpec): Promise<ListData<Item>> {
		return SimpleHTTP.newRequest(
			kSpreadsheetURL,
			{interpretResponse: true}
		).get<CSVDataItem[]>().then(response => {
			const result: ListData<Item> = {
				items: []
			};
			for (const csv of response.interpreted) {
				result.items.push(new Item(csv));
			}
			this.lastItems = result.items;	// Hold on to those to any details request
			return result;
		});
	}

	/* Since all data is provided by the getList call, we don't
	 * expect any getDetails call, but need to implement one anyway to make everyone happy.
	 */
	getDetails(spec: FeedDetailsSpec): Promise<Item | undefined> {
		return Promise.resolve(this.lastItems[spec.index]);
	}
}

interface CSVDataItem {
	"Title": string;
	"Given Name": string;
	"Surname": string;
	"City": string;
	"Country": string;
	"Company": string;
}

/**
 * I use same data type both in lists and item views.
 */
class Item {

	@field("Mr, Ms, Dr, etc")
	readonly salutation: string;

	@field("First name")
	readonly givenName: string;

	@field("Last name")
	readonly surName: string;

	@field()
	readonly city: string;

	@field()
	readonly coutry: string;

	@field()
	readonly company: string;


	constructor(csv: CSVDataItem) {
    
		this.salutation = csv.Title;
		this.givenName = csv['Given Name'];
		this.surName =   csv.Surname;
		this.city = csv.City;
		this.coutry = csv.Country;
		this.company = csv.Company;
	}
}
